const crypto = require('crypto');
const fse = require('fs-extra');
const _ = require('lodash');

const DATA_PATH = './data/comprehension_snippets.json';
const EDITABLE_SNIPPETS = './data/editable_snippets.json';
const FIXME_SNIPPETS = './data/repair_snippets.json';
const COMPLETED_PATH = './data/completed.json';
const COMPENSATED_PATH = './data/compensated.json';

const MAX_EDITABLE = 1;
const MAX_REPAIR = 1;

const getUserData = sessionId => {
    const hashedId = crypto.createHash('sha1').update(sessionId).digest('base64');
    const dataPath = '/tmp/'+hashedId+'.json';

    // User file should already exist
    // TODO: if not, route to signup
    if (!fse.pathExistsSync(dataPath)) {
        return false;
    }

    return fse.readJsonSync(dataPath);
}

const createUser = (sessionId, userType) => {
    const hashedId = crypto.createHash('sha1').update(sessionId).digest('base64');
    const dataPath = '/tmp/'+hashedId+'.json';

    const comprehensionSnippets = fse.readJsonSync(DATA_PATH);

    // Comprehension Questions
    const filteredSnippets = _.map(comprehensionSnippets, (snippet) => {
        const commentType = Math.random() < 0.5 ? 'human' : 'machine';
        let newSnippet = _.omit(snippet, ['comments']);
        newSnippet.comment = { type: commentType, text: snippet.comments[commentType] }
        newSnippet.relevantSummaries = _.reduce(
            newSnippet.relevantSummaries,
            (acc, summary, key) => {
                acc[key] = { type: commentType, text: summary[commentType] }
                return acc;
            }, {}
        );
        return newSnippet;
    });
    const shuffledComprehensionSnippets = _.shuffle(filteredSnippets);

    // Method write questions
    const editableSnippets = fse.readJsonSync(EDITABLE_SNIPPETS);
    const filteredEditable = _.map(editableSnippets, (snippet) => {
        const commentType = Math.random() < 0.5 ? 'human' : 'machine';
        let newSnippet = _.omit(snippet, ['snippets']);
        newSnippet.snippet = snippet.snippets[commentType];
        newSnippet.label = commentType;
        newSnippet.questions = _.reduce(
            newSnippet.questions,
            (acc, question) => {
                if (!_.isPlainObject(question.data)) {
                    acc.push(question);
                    return acc;
                }
                let newQuestion = _.cloneDeep(question);
                newQuestion.data = newQuestion.data[commentType];
                acc.push(newQuestion);
                return acc;
        }, []);
        return newSnippet;
    });
    const shuffledEditableSnippets = _.shuffle(filteredEditable);
    const chosenEditableSnippets = _.slice(shuffledEditableSnippets, 0, MAX_EDITABLE);

    const usedClasses = new Set();
    _.forEach(chosenEditableSnippets, snippet => {
        usedClasses.add(snippet.class);
    });

    // Bugfix questions
    const repairSnippets = fse.readJsonSync(FIXME_SNIPPETS);
    const noRepeatSnippets = _.reject(repairSnippets, snippet => usedClasses.has(snippet.class));
    const filteredRepair = _.map(noRepeatSnippets, (snippet) => {
        const commentType = Math.random() < 0.5 ? 'human' : 'machine';
        let newSnippet = _.omit(snippet, ['snippets']);
        newSnippet.snippet = snippet.snippets[commentType];
        newSnippet.label = commentType;
        newSnippet.questions = _.reduce(
            newSnippet.questions,
            (acc, question) => {
                if (!_.isPlainObject(question.data)) {
                    acc.push(question);
                    return acc;
                }
                let newQuestion = _.cloneDeep(question);
                newQuestion.data = newQuestion.data[commentType];
                acc.push(newQuestion);
                return acc;
        }, []);
        return newSnippet;
    });
    const shuffledRepairSnippets = _.shuffle(filteredRepair);
    const chosenRepairSnippets = _.slice(shuffledRepairSnippets, 0, MAX_REPAIR);

    const snippets = {
        comprehension: shuffledComprehensionSnippets,
        methodWrite: chosenEditableSnippets,
        repair: chosenRepairSnippets
    }

    let userData = {
        id: hashedId,
        surveyStart: Date.now(),
        userType,
        sections: ['comprehension', 'methodWrite', 'repair'],
        sectionIdx: 0,
        snippetIdx: 0,
        questionIdx: 0,
        snippets,
        responses: { 'comprehension': {}, 'methodWrite': {}, 'repair': {}}
    }
    fse.outputJsonSync(dataPath, userData);
    return userData;
}

const updateUser = (newUserData) => {
    if (!newUserData.id) return false;
    const dataPath = '/tmp/'+newUserData.id+'.json';

    fse.outputJsonSync(dataPath, newUserData, { spaces: 2 });
    return newUserData;
}

const markCompleted = (sessionId, ip) => {
    const hashedId = crypto.createHash('sha1').update(sessionId).digest('base64');
    const completed = fse.readJsonSync(COMPLETED_PATH);
    if (completed.sessionIds.includes(hashedId)) {
        return true;
    }
    completed.sessionIds.push(hashedId);

    if (ip) {
        const hashedIp = crypto.createHash('sha1').update(ip).digest('base64');
        if (!completed.ips.includes(hashedIp)) {
            completed.ips.push(hashedIp);
        }
    }
    
    fse.outputJsonSync(COMPLETED_PATH, completed);
    return true;
}

const checkCompleted = (sessionId, ip) => {
    const hashedId = crypto.createHash('sha1').update(sessionId).digest('base64');
    const completed = fse.readJsonSync(COMPLETED_PATH);
    const idPresent = completed.sessionIds.includes(hashedId);

    const hashedIp = ip ? crypto.createHash('sha1').update(ip).digest('base64') : undefined;
    const ipPresent = ip && completed.ips.includes(hashedIp);

    return idPresent || ipPresent;
}

const checkCompensated = sessionId => {
    const hashedId = crypto.createHash('sha1').update(sessionId).digest('base64');
    const compensated = fse.readJsonSync(COMPENSATED_PATH);
    return compensated.collected.includes(hashedId);
}

const recordCompensation = (sessionId, email) => {
    const hashedId = crypto.createHash('sha1').update(sessionId).digest('base64');
    const compensated = fse.readJsonSync(COMPENSATED_PATH);
    compensated.collected.push(hashedId);
    compensated.emails.push(email);
    fse.outputJsonSync(COMPENSATED_PATH, compensated);
    return true;
}

module.exports = {
    checkCompensated,
    checkCompleted,
    createUser,
    getUserData,
    markCompleted,
    recordCompensation,
    updateUser
}