import { Component } from 'preact';
import { route } from 'preact-router';
import style from '../routes/snippet/style';
import TemplateResponse from './template';
import axios from 'axios';
import { getCookie } from './utils';
import { UnControlled as CodeMirror} from 'react-codemirror2'
require('codemirror/mode/clike/clike');
require('codemirror/lib/codemirror.css');
require('codemirror/theme/material.css');

export default class TaskBasedSnippet extends Component {
    constructor(props) {
        super();
        this.state = {
            fid: -1,
            snippetIdx: -1,
            body: '',
            template: '',
			questionAnswer: '',
			question: {},
			snippet: '',
			codeResponses: [],
			submittedOnce: false,
			submissionsLeft: 5
		};
		this.handleTemplateChange = this.handleTemplateChange.bind(this);
		this.handleTextChange = this.handleTextChange.bind(this);
        this.submitQuestion = this.submitQuestion.bind(this);
        this.incrementSnippet = this.incrementSnippet.bind(this);
		this.reloadState = this.reloadState.bind(this);
		this.submitCode = this.submitCode.bind(this);
	}

	componentDidMount() {
		this.reloadState();
	}

	reloadState() {
		const sessionId = getCookie('session_id');
		if (sessionId) {
			axios.post('/api/userStatus', { sessionId }).then(result => {
				if (result.data && result.data.success) {
                    const { snippetIdx, response, question } = result.data;
					const { fid, snippet } = result.data.data;
					// Since we can't go back, we can assume the most recent answer is the most relevant
					let questionAnswer = '';
					if (question.type === 'template') {
						questionAnswer = question.data;
					}
					
					this.setState({ fid, snippet, snippetIdx, questionAnswer, question });
				} else {
					route('/', true);
				}
			});
		} else {
			route('/', true);
		}
	}
	
	incrementSnippet() {
		const sessionId = getCookie('session_id');
		axios.post('/api/nextQuestion', { sessionId }).then(result => {
			if (result.data && result.data.success) {
				const { snippetIdx, response, section, question } = result.data;
				this.setState({
					submissionsLeft: 5,
					submittedOnce: false
				});
				if (section != this.props.section) {
					this.props.reloadState();
					this.reloadState();
				} else {
					// Since we can't go back, we can assume the most recent answer is the most relevant
					const questionAnswer = response && response.questions
						? response.questions[response.questions.length-1].answer : question.type == 'template' ? question.data : '';
					const { fid, snippet, comment, relevantSummaries } = result.data.data;
					this.setState({ fid, snippet, snippetIdx, comment, relevantSummaries, questionAnswer, question });
				}
			}
		});
	}
	
	submitQuestion(e) {
		e.preventDefault();
		const { fid, questionAnswer, snippetIdx, template, question } = this.state;
		const sessionId = getCookie('session_id');

		const postData = {
			fid,
			answer: questionAnswer,
			sessionId,
			template,
			questionKey: question.data
		}
		return axios.post(`/api/snippet/${this.props.section}/${snippetIdx}`, postData).then(result => {
			if (result.data && result.data.success) {
				this.setState({ questionAnswer: '', codeResponses: [], apiError: '' });
				this.incrementSnippet();
			} else {
				alert('Something went wrong.');
			}
		});
	}

	submitCode(e) {
		e.preventDefault();
		this.setState({ codeResponses: [] });
		const { fid, snippetIdx, question, questionAnswer } = this.state;
		const sessionId = getCookie('session_id');
		const postData = {
			fid,
			code: questionAnswer,
			sessionId,
			questionKey: question.data
		}
		this.setState({ loading: true });
		return axios.post(`/api/code/${this.props.section}/${snippetIdx}`, postData).then(result => {
			// result.data is an array of [{ pass: true/false, output: system output }]
			if (result.data && result.data.success) {
				let numSuccesses = 0;
				for (let i = 0; i < result.data.data.length; i++) {
					if (result.data.data[i].pass) {
						numSuccesses += 1;
					}
				}
				const attempts = result.data.attempts;
				this.setState({
					passed: !!numSuccesses,
					codeResponses: result.data.data,
					loading: false,
					submittedOnce: true,
					submissionsLeft: attempts[1] - attempts[0]
				});
			} else if (result.data && !result.data.success) {
				this.setState({
					loading: false,
					apiError: result.data.err
				});
			}
		}).catch(err => {
			console.log('error:', err);
		});
	}

    handleTemplateChange(editor, data, value) {
		this.setState({ questionAnswer: value });
	}
	
	handleTextChange(e) {
		this.setState({ questionAnswer: e.target.value });
	}

    render() {
        const {
			apiError,
			body,
			codeResponses,
			loading,
            snippetIdx,
            snippet,
			question
		} = this.state;
		let blocks = [];
		if (question && question.data) {
			blocks = question.data.split('```');
		}

		return (
			<div class={style.profile}>
				<h1>{this.props.section} Snippet {snippetIdx + 1}</h1>
				<p>
					Spend some time reading through the below project, then follow the instructions below.
				</p>
				<div class={style.mainContentWrapper}>
					<CodeMirror
						value={snippet.replace('${userResponse}\n', '')}
						options={{
							mode: 'text/x-java',
							theme: 'material',
							gutter: false,
							lineNumbers: true,
							readOnly: true
						}}
					/>
					{ question.type == 'template' ? 
						<TemplateResponse
							value={this.state.questionAnswer}
							onChange={this.handleTemplateChange}
							onSubmit={this.submitQuestion}
							runCode={this.submitCode}
							passed={this.state.passed}
							codeResponses={codeResponses}
							loading={loading}
							setRef={this.setCodeRef}
							onChange={this.handleTemplateChange}
							apiError={apiError}
							question={question}
							submittedOnce={this.state.submittedOnce}
							submissionsLeft={this.state.submissionsLeft}
						/> : (<form onSubmit={this.submitQuestion}>
							<div class={style.questionWrapper}>
								<h2>{blocks.map((block, idx) => (idx % 2 == 0 ? block : (<span class={style.inlineCode}>{block}</span>)))}</h2>
								{ question.type == 'question' ?
									(<textarea required class={style.question} value={this.state.questionAnswer} onChange={this.handleTextChange}></textarea>) :
									question.type == 'radio' ? question.options.map(elt => (
										<div><input type='radio' name='question' value={elt.value} required={true} onChange={this.handleTextChange}></input>{elt.label}<br/></div>
									)) : '' }
							</div>
							<button type="submit" class={style.button}>Next Question</button>
						</form>)
					}
				</div>
			</div>
		);
    }
}